'use strict';

module.exports = function (gulp, plugins, config, name, file) { // eslint-disable-line func-names
  const theme = config.themes[name],
    srcBase = config.projectPath + 'var/view_preprocessed/soul' + theme.dest.replace('pub/static', ''),
    stylesDir = theme.stylesDir ? theme.stylesDir : 'styles',
    dest = [],
    disableMaps = plugins.util.env.disableMaps || false,
    production = plugins.util.env.prod || false,
    postcss = [],
    disableSuffix = theme.disableSuffix || false,
    browserslist = require('../helper/config-loader')('browserslist.json', plugins, config);

  plugins.path = require('path');
  plugins.helper.sassLint = require('../helper/sass-lint');

  postcss.push(
    plugins.autoprefixer({
      browsers: browserslist,
      grid: true
    }),
    plugins.cssnano({
      discardComments: {
        removeAll: true
      },
      minimiseWhitespace: true,
      discardDuplicates: true,
      colormin: true,
      discardOverridden: true,
      functionOptimiser: true,
      zindex: false
    })
  );

  function adjustDestinationDirectory(file) {
    if (file.dirname.startsWith(stylesDir)) {
      file.dirname = file.dirname.replace(stylesDir, 'css');
    } else {
      file.dirname = file.dirname.replace('/' + stylesDir, '');
    }
    return file;
  }

  theme.locale.forEach(locale => {
    dest.push(config.projectPath + theme.dest + '/' + locale);
  });
  var replace = require('gulp-string-replace');

  return gulp.src(
    file || srcBase + '/**/*.scss', {
      base: srcBase
    }
  )
  .pipe(
    plugins.if(
      !plugins.util.env.ci,
      plugins.plumber()
    )
  )
  .pipe(plugins.if(!disableMaps, plugins.sourcemaps.init()))
  .pipe(
    plugins.sass()
    .on('error', plugins.sassError.gulpSassError(plugins.util.env.ci || false))
  )
  .pipe(plugins.if(postcss.length, plugins.postcss(postcss || [])))
  .pipe(replace(new RegExp('@charset "UTF-8";', 'g'), ''))
  .pipe(plugins.if(production && !disableSuffix, plugins.rename({
    suffix: '.min'
  })))
  .pipe(plugins.if(!disableMaps, plugins.sourcemaps.write('.', {
    includeContent: true
  })))
  .pipe(plugins.rename(adjustDestinationDirectory))
  .pipe(plugins.multiDest(dest))
  .pipe(plugins.helper.sassLint(gulp,plugins,config))
  .pipe(plugins.browserSync.stream());
};
